const ServiceConfig = require("./ServiceConfig");
const Calendar = require("./Calendar");
const DeepCopy = require("../Utils/DeepCopy");

class MenuAnalytics {
  static data = {
    dishCounter: {},
    groupCounter: {},
  };

  get data (){
    return DeepCopy.execute(MenuAnalytics.data);
  }

  set data(analytic) {
    MenuAnalytics.data = analytic;
  }

  clear() {
    MenuAnalytics.data = {
      dishCounter: {},
      groupCounter: {},
    };
  }

  hasLocalCharacteristic(structureToFind, itemToFind, typeDish) {
    if(!itemToFind || !structureToFind || !structureToFind[typeDish]) return false;
    if(!(structureToFind[typeDish] instanceof Set)) debugger;
    return structureToFind[typeDish].has(itemToFind);
  }

  hasGlobalCharacteristic(structureToFind, itemToFind) {
    if(!itemToFind || !structureToFind || !structureToFind.all) return false;
    return structureToFind.all.has(itemToFind);
  }

  getAnalytic(menuDate) {
    return MenuAnalytics.data[menuDate];
  }

  getAnalyticConsecutiveDays(menuDate) {
    const curDateIdx = Calendar.dates.findIndex(date => date === menuDate);
    if(curDateIdx === -1) return { yesterday: null, tomorrow: null };
    const yesterday = curDateIdx > 0 ? Calendar.dates[curDateIdx - 1] : null;
    const tomorrow = curDateIdx < Calendar.dates.length - 1 ? Calendar.dates[curDateIdx + 1] : null;
    return {
      yesterday: this.getAnalytic(yesterday),
      tomorrow: this.getAnalytic(tomorrow),
    }
  }

  isValidDishInConsecutiveDays(menuDate, sequential, dish) {
    if(ServiceConfig.hasIngredientValidation(sequential)) {
      const { yesterday, tomorrow } = this.getAnalyticConsecutiveDays(menuDate);
      const yesterdayIngr = yesterday && yesterday.ingredient[dish.COD_TIP_PRATO] ? yesterday.ingredient[dish.COD_TIP_PRATO] : new Set();
      const tomorrowIngr = tomorrow && tomorrow.ingredient[dish.COD_TIP_PRATO] ? tomorrow.ingredient[dish.COD_TIP_PRATO] : new Set();
      if(yesterdayIngr.has(dish.INGR_PRINCIPAL_1) || yesterdayIngr.has(dish.INGR_PRINCIPAL_2)) return false;
      if(tomorrowIngr.has(dish.INGR_PRINCIPAL_1) || tomorrowIngr.has(dish.INGR_PRINCIPAL_2)) return false;
    }
    return true;
  }

  isValidDishOnDay(menuDate, sequential, dish) {
    const {
      isFixColumn,
      ignoreColor,
      ignorePrepMode
    } = ServiceConfig.getConfig(sequential);
    if(!MenuAnalytics.data[menuDate]) return true;
    const {
      color,
      prepMode,
      ingredient,
    } = MenuAnalytics.data[menuDate];
    const { COR_PRATO, MODO_PREPARO, COD_TIP_PRATO, INGR_PRINCIPAL_1, INGR_PRINCIPAL_2 } = dish;
    if(isFixColumn) return true;
    if(!ignoreColor && this.hasLocalCharacteristic(color, COR_PRATO, COD_TIP_PRATO)) {
      return false;
    }
    if(!ignorePrepMode && this.hasLocalCharacteristic(prepMode, MODO_PREPARO, COD_TIP_PRATO)) {
      return false;
    }
    if(this.hasGlobalCharacteristic(ingredient, INGR_PRINCIPAL_1)) return false;
    if(this.hasGlobalCharacteristic(ingredient, INGR_PRINCIPAL_2)) return false;
    return true;
  }

  filterPreferenceDishesOnDay(menuDate, sequential, dishes) {
    const {
      isFixColumn = false,
    } = ServiceConfig.getConfig(sequential) || {};
    const curDateIdx = Calendar.dates.findIndex(date => date === menuDate);
    const { groups, typeCut } = this.getAnalytic(menuDate) || {};
    let yesterdayGroups = {};
    let tomorrowGroups = {};
    if(curDateIdx > 0) {
      const yesterdayDate = Calendar.dates[curDateIdx - 1];
      const yesterdayAnalytic = this.getAnalytic(yesterdayDate);
      yesterdayGroups = yesterdayAnalytic ? yesterdayAnalytic.groups : {};
    }
    if(curDateIdx < Calendar.dates.length - 1) {
      const tomorrowDate = Calendar.dates[curDateIdx + 1];
      const tomorrowAnalytic = this.getAnalytic(tomorrowDate);
      tomorrowGroups = tomorrowAnalytic ? tomorrowAnalytic.groups : {};
    }
    if(isFixColumn || !groups || !typeCut) return dishes;
    return dishes.filter(dish => {
      if(groups[dish.COD_TIP_PRATO] && groups[dish.COD_TIP_PRATO].has(dish.gpprato)) {
        return false;
      }
      if(yesterdayGroups[dish.COD_TIP_PRATO] && yesterdayGroups[dish.COD_TIP_PRATO].has(dish.gpprato)) {
        return false;
      }
      if(tomorrowGroups[dish.COD_TIP_PRATO] && tomorrowGroups[dish.COD_TIP_PRATO].has(dish.gpprato)) {
        return false;
      }
      if(typeCut.all.has(dish.TP_CORTE)) {
        return false;
      }
      return true;
    });
  }

  addColor(menuDate, dish) {
    const { color } = MenuAnalytics.data[menuDate];
    const { COR_PRATO, COD_TIP_PRATO } = dish;
    color.all.add(COR_PRATO);
    color[COD_TIP_PRATO] = color[COD_TIP_PRATO] || new Set();
    color[COD_TIP_PRATO].add(COR_PRATO);
  }

  addPrepMode(menuDate, dish) {
    const { prepMode } = MenuAnalytics.data[menuDate];
    const { MODO_PREPARO, COD_TIP_PRATO } = dish;
    prepMode.all.add(MODO_PREPARO);
    prepMode[COD_TIP_PRATO] = prepMode[COD_TIP_PRATO] || new Set();
    prepMode[COD_TIP_PRATO].add(MODO_PREPARO);
  }

  addIngredient(menuDate, dish) {
    const { ingredient } = MenuAnalytics.data[menuDate];
    const { COD_TIP_PRATO, INGR_PRINCIPAL_1, INGR_PRINCIPAL_2 } = dish;
    ingredient[COD_TIP_PRATO] = ingredient[COD_TIP_PRATO] || new Set();
    if(INGR_PRINCIPAL_1) {
      ingredient.all.add(INGR_PRINCIPAL_1);
      ingredient[COD_TIP_PRATO].add(INGR_PRINCIPAL_1);
    }
    if(INGR_PRINCIPAL_2) {
      ingredient.all.add(INGR_PRINCIPAL_2);
      ingredient[COD_TIP_PRATO].add(INGR_PRINCIPAL_2);
    }
  }

  addTypeCut(menuDate, dish) {
    const { typeCut } = MenuAnalytics.data[menuDate];
    const { TP_CORTE } = dish;
    typeCut.all.add(TP_CORTE);
  }

  addGroup(menuDate, dish) {
    const { groups } = MenuAnalytics.data[menuDate];
    const { gpprato, COD_TIP_PRATO } = dish;
    groups.all.add(gpprato);
    groups[COD_TIP_PRATO] = groups[COD_TIP_PRATO] || new Set();
    groups[COD_TIP_PRATO].add(gpprato);
  }

  addDishAndGroupUtilization(dish) {
    MenuAnalytics.data.dishCounter[dish.prato] = MenuAnalytics.data.dishCounter[dish.prato] || 0;
    MenuAnalytics.data.groupCounter[dish.gpprato] = MenuAnalytics.data.groupCounter[dish.gpprato] || 0;
    MenuAnalytics.data.dishCounter[dish.prato]++;
    MenuAnalytics.data.groupCounter[dish.gpprato]++;
  }

  analyze(menuDate, sequential, dish) {
    MenuAnalytics.data[menuDate] = MenuAnalytics.data[menuDate] || {
      ingredient: { all: new Set() },
      color:      { all: new Set() },
      typeCut:    { all: new Set() },
      prepMode:   { all: new Set() },
      groups:     { all: new Set() },
    };
    const {
      isFixColumn = false,
      ignoreColor = false,
      ignorePrepMode = false,
    } = ServiceConfig.getConfig(sequential);
    if(!isFixColumn) {
      this.addIngredient(menuDate, dish);
      if(!ignoreColor && dish.COR_PRATO) {
        this.addColor(menuDate, dish);
      }
      if(!ignorePrepMode && dish.MODO_PREPARO) {
        this.addPrepMode(menuDate, dish);
      }
      if(dish.TP_CORTE) {
        this.addTypeCut(menuDate, dish);
      }
    }
    this.addGroup(menuDate, dish);
    this.addDishAndGroupUtilization(dish);
  }
}

module.exports = new MenuAnalytics();
